import json
import os

from django.apps import apps
from django.core.management import call_command
from django.core.serializers import serialize
from django.conf import settings

default_db_seed = {
    'seed_dir': f"{settings.BASE_DIR}/db_seed/",
    'apps': [
        "admin",
        "auth",
    ]
}

db_seed = getattr(
    settings,
    'DJANGO_DB_SEED',
    default_db_seed
)

db_seed_dir = db_seed.get('seed_dir', default_db_seed['seed_dir'])
db_seed_apps = db_seed.get('apps', default_db_seed['apps'])


def get_models():
    """
    Get a list of all models in the database
    """

    models = {}

    # include only these apps to back up their data
    app_names = db_seed_apps

    # Get a list of all installed apps
    installed_apps = apps.get_app_configs()

    # Iterate over each app and list its models
    for app in installed_apps:
        if app.label not in models:
            _models = app.get_models()

            if app.label not in app_names:
                continue

            models[app.label] = []
            for model in _models:
                models[app.label].append(f"{model._meta.object_name}")

    return models


def backup():
    """
    Backup data from the database
    """

    models = get_models()

    # loop over models
    for app in models:
        print(f"App: {app}")
        ensure_directory_exists(db_seed_dir + app)
        for model in models[app]:
            print(f" - {model}")
            # get the model's data
            model_class = apps.get_model(app, model)
            model_data = model_class.objects.all()

            print(f"   - backing up {model}...")

            # dumpdata
            dump_model_data_to_json(f"{app}.{model}", f"{db_seed_dir}{app}/{model}.json")


def restore():
    """
    Restore data from a backup
    """
    models = get_models()

    # loop over models
    for app in models:
        print(f"\n\rApp: {app}")
        for model in models[app]:
            print(f" - {model}")
            # restore the model
            print(f"   - restoring {model}...")
            # restore the model
            model_class = apps.get_model(app, model)
            truncate_data(model_class)

            # loaddata
            load_data_from_json(f"{db_seed_dir}{app}/{model}.json")


def ensure_directory_exists(directory):
    if not os.path.exists(directory):
        os.makedirs(directory)


def dump_model_data_to_json_stringify(model_name, output_file):
    # Call Django's dumpdata management command to dump data from the specified model
    with open(output_file, 'w') as f:
        call_command('dumpdata', model_name, stdout=f)


def dump_model_data_to_json(model_name, output_file, indent=4):
    # Get the model class dynamically
    model_class = apps.get_model(model_name)

    # Query data from the model
    queryset = model_class.objects.all()

    # Serialize queryset to JSON
    serialized_data = serialize('json', queryset)

    # Load serialized data
    loaded_data = json.loads(serialized_data)

    # Write the JSON data to a file
    with open(output_file, 'w') as f:
        json.dump(loaded_data, f, indent=indent)


def load_data_from_json(input_file):
    # Call Django's loaddata management command to load data from the specified JSON file
    with open(input_file, 'r') as f:
        data = json.load(f)
        call_command('loaddata', input_file)


def truncate_data(model_class):
    # Truncate existing data in the model
    model_class.objects.all().delete()

    # Your restoration logic here
    # For example, you can load data from a JSON file and create model instances
    # After truncating, you can seed data or load data from a file
    # Example: data = [...] and then create instances

    print('Truncated data from the model:', model_class)



from setuptools import setup, find_packages

setup(
    name='django-db-seed',
    version='1.0.0',
    packages=[
        'django_db_seed',
        'django_db_seed.management',
        'django_db_seed.management.commands',
        'django_db_seed.migrations',
    ],
    include_package_data=True,
    package_data={
        'django_db_seed': ['management/commands/*.py'],
    },
    install_requires=[
        'Django>=5.0.3',
        # Add any other dependencies here
    ],
    setup_requires=['wheel'],
    python_requires='>=3.12',
    license='MIT',
    description='A Django app to backup and restore the database.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/AfzalSabbir/django_db_seed',
    author='Afzalur Rahman Sabbir',
    author_email='afzalbd1@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 5.0',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.core.management.base import BaseCommand

from django_db_seed.helpers import backup, restore


class Command(BaseCommand):
    help = 'This command will backup/restore the database.'

    def add_arguments(self, parser):
        # Add a positional argument
        parser.add_argument('--mode', type=str, help='Mode: backup or restore')

    def handle(self, *args, **options):
        mode = options['mode']

        if mode == 'backup' or mode == 'create' or mode is None:
            backup()
        elif mode == 'restore' or mode == 'load':
            restore()
        else:
            self.stdout.write(self.style.ERROR('Invalid mode!'))
